#include <bits/stdc++.h>
using namespace std;

typedef vector< vector<int> > rooted_tree;
typedef vector< vector<int> > tree;

vector<int> find_longest_path(const tree &T, int start) {
    vector<int> dist(T.size(), 987654321), from(T.size(), -1);
    queue<int> Q;
    Q.push(start);
    dist[start] = 0;
    int last = start;
    while (!Q.empty()) {
        int where = Q.front(); Q.pop();
        for (int v : T[where]) {
            if (dist[v] <= dist[where]+1) continue;
            dist[v] = dist[where]+1;
            from[v] = where;
            Q.push(v);
            last = v;
        }
    }
    vector<int> answer;
    while (true) {
        answer.push_back(last);
        last = from[last];
        if (last == -1) break;
    }
    reverse( answer.begin(), answer.end() );
    return answer;
}

vector<int> find_centers(const tree &T) {
    vector<int> path1 = find_longest_path(T,0);
    if (path1 == vector<int>{-1}) return {};
    vector<int> path2 = find_longest_path(T,path1.back());
    int L = path2.size();
    if (L % 2 == 0) {
        return { path2[L/2 - 1], path2[L/2] };
    } else {
        return { path2[L/2] };
    }
}

bool is_tree(const tree &T) {
    return !find_centers(T).empty();
}

rooted_tree root_tree(const tree &T, int root) {
    rooted_tree R(T.size());
    vector<int> dist(T.size(), 987654321);
    queue<int> Q;
    Q.push(root);
    dist[root] = 0;
    while (!Q.empty()) {
        int where = Q.front(); Q.pop();
        for (int v : T[where]) {
            if (dist[v] <= dist[where]+1) continue;
            dist[v] = dist[where]+1;
            R[where].push_back(v);
            Q.push(v);
        }
    }
    return R;
}

void dfs_label(const rooted_tree &R, map< vector<int>, int> &label_map, vector<int> &labels, int v) {
    vector<int> children;
    for (int w : R[v]) {
        dfs_label(R,label_map,labels,w);
        children.push_back( labels[w] );
    }
    sort( children.begin(), children.end() );
    if (!label_map.count(children)) {
        int new_label = label_map.size();
        label_map[children] = new_label;
    }
    labels[v] = label_map[children];
}

bool are_isomorphic_rooted(const rooted_tree &R1, int root1, const rooted_tree &R2, int root2) {
    map< vector<int>, int > label_map;
    vector<int> labels1( R1.size(), -1 ), labels2( R2.size(), -1 );
    dfs_label(R1, label_map, labels1, root1);
    dfs_label(R2, label_map, labels2, root2);
    return labels1[root1] == labels2[root2];
}

bool are_isomorphic(const tree &T1, const tree &T2) {
    for (int r1 : find_centers(T1)) for (int r2 : find_centers(T2)) {
        rooted_tree R1 = root_tree(T1,r1), R2 = root_tree(T2,r2);
        if (are_isomorphic_rooted(R1,r1,R2,r2)) return true;
    }
    return false;
}

void error(const string &msg) {
    cout << "0.0" << endl;
    // cerr << "translate:wrong" << endl;
    cerr << msg << endl;
    exit(0);
}

void bug(const string &msg) {
    cerr << msg << endl;
    exit(1);
}

int main(int argc, char** argv) {
    if (argc < 4) bug("usage: ./checker input our_output contestants_output");
    ifstream fin(argv[1]), fout(argv[2]), fcont(argv[3]);
    if (!fin) bug("input file missing");
    if (!fout) bug("our output file missing");
    if (!fcont) bug("contestant's output file missing");

    string line;
    
    int T;
    getline(fin,line);
    stringstream(line) >> T;

    int N;
    getline(fin,line);
    stringstream(line) >> N;

    vector<int> D(N);
    getline(fin,line);
    { stringstream ss(line); for (int n=0; n<N; ++n) ss >> D[n]; }

    int ourK;
    getline(fout,line);
    stringstream(line) >> ourK;

    int theirK;
    getline(fcont,line);
    if (!fcont) error("Nepodařilo se přečíst první řádek vstupu.");
    { stringstream ss(line); if (!(ss >> theirK)) error("Nepodařilo se přečíst k."); }

    if (ourK != theirK) error("Chybné k.");
    if (T == 1) {
	cout << "1.0" << endl;
	cerr << "translate:success" << endl;
        return 0;
    }

    vector<tree> TS(theirK);
    for (int k=0; k<theirK; ++k) {
        TS[k].resize(N);
        vector<int> stupne(N,0);
        for (int n=0; n<N-1; ++n) {
            int x, y;
            if (!(fcont >> x >> y)) error("Nepodařilo se přečíst popis mapy.");
            if (x < 1 || x > N || y < 1 || y > N) error("Číslo ostrova mimo rozsah 1-n.");
            if (x == y) error("Most spojující ostrov se sebou samým.");
            --x;
            --y;
            TS[k][x].push_back(y);
            TS[k][y].push_back(x);
            ++stupne[x];
            ++stupne[y];
        }
        if (D != stupne) error("Počtu mostů vedoucí z ostrovů neodpovídají vstupu.");
        if (!is_tree(TS[k])) error("Mosty na mapě nemají stromovu topologii.");
    }

    if (theirK == 2) if (are_isomorphic(TS[0],TS[1])) error("Obě vypsané mapy jsou stejné.");

    if (fcont >> line) error("Příliš mnoho údajů ve výstupu - za poslední mapou už nesmí nic být.");

    cout << "1.0" << endl;
    cerr << "translate:success" << endl;
    return 0;
}
